@@ -87,6 +87,10 @@ pub struct Initial {
     config: Option<Value>,
     #[serde(default = "default_workers")]
     workers: u32,
+    #[serde(default)]
+    user: Option<String>,
+    #[serde(default)]
+    group: Option<String>,
 }
 
 impl Initial {
@@ -102,6 +106,8 @@ impl Initial {
         bus: BusConfig,
         config: Option<&Value>,
         workers: u32,
+        user: Option<&str>,
+        group: Option<&str>,
     ) -> Self {
         Self {
             config_version: SERVICE_CONFIG_VERSION,
@@ -115,6 +121,8 @@ impl Initial {
             bus,
             config: config.map(Clone::clone),
             workers,
+            user: user.map(ToOwned::to_owned),
+            group: group.map(ToOwned::to_owned),
         }
     }
     #[inline]
@@ -138,6 +146,14 @@ impl Initial {
         self.prepare_command.as_deref()
     }
     #[inline]
+    pub fn user(&self) -> Option<&str> {
+        self.user.as_deref()
+    }
+    #[inline]
+    pub fn group(&self) -> Option<&str> {
+        self.group.as_deref()
+    }
+    #[inline]
     pub fn data_path(&self) -> &str {
         &self.data_path
     }
@@ -272,12 +288,69 @@ impl Initial {
         let rpc = RpcClient::create(bus, handlers, opts);
         Ok(Arc::new(rpc))
     }
+    #[inline]
     pub fn init_registry(&self, rpc: &Arc<RpcClient>) -> Registry {
         Registry {
             id: self.id.clone(),
             rpc: rpc.clone(),
         }
     }
+    /// Drops current privileges to user/group if set (must be started under root)
+    ///
+    /// Note: if the service group is set to "nogroup" and the group is not found, the method
+    /// automatically tries to locate the group "nobody"
+    #[inline]
+    pub fn drop_privileges(&self) -> EResult<()> {
+        if let Some(ref group) = self.group {
+            let g = match get_system_group(group) {
+                Ok(v) => v,
+                Err(e) if group == "nogroup" => {
+                    if let Ok(v) = get_system_group("nobody") {
+                        v
+                    } else {
+                        return Err(e);
+                    }
+                }
+                Err(e) => return Err(e),
+            };
+            nix::unistd::setgroups(&[g.gid]).map_err(|e| {
+                Error::failed(format!(
+                    "Failed to switch the process group to {}: {}",
+                    group, e
+                ))
+            })?;
+            nix::unistd::setgid(g.gid).map_err(|e| {
+                Error::failed(format!(
+                    "Failed to switch the process group to {}: {}",
+                    group, e
+                ))
+            })?;
+        }
+        if let Some(ref user) = self.user {
+            let u = get_system_user(user)?;
+            nix::unistd::setuid(u.uid).map_err(|e| {
+                Error::failed(format!(
+                    "Failed to switch the process user to {}: {}",
+                    user, e
+                ))
+            })?;
+        }
+        Ok(())
+    }
+}
+
+fn get_system_user(user: &str) -> EResult<nix::unistd::User> {
+    let u = nix::unistd::User::from_name(user)
+        .map_err(|e| Error::failed(format!("failed to get the system user {}: {}", user, e)))?
+        .ok_or_else(|| Error::failed(format!("Failed to locate the system user {}", user)))?;
+    Ok(u)
+}
+
+fn get_system_group(group: &str) -> EResult<nix::unistd::Group> {
+    let g = nix::unistd::Group::from_name(group)
+        .map_err(|e| Error::failed(format!("failed to get the system group {}: {}", group, e)))?
+        .ok_or_else(|| Error::failed(format!("Failed to locate the system group {}", group)))?;
+    Ok(g)
 }
 
 #[derive(Debug, Serialize, Deserialize, Clone, Default)]
