@@ -540,7 +540,7 @@ async fn extend_string_value(val: String, op: &crate::op::Op, base: &Path) -> ER
                 path.push(fname);
                 let content =
                     tokio::time::timeout(op.timeout()?, tokio::fs::read_to_string(path)).await??;
-                Ok(Value::String(content))
+                Ok(Value::String(content.trim_end().to_string()))
             }
             "pipe" => {
                 let s = pipe!();
@@ -549,7 +549,7 @@ async fn extend_string_value(val: String, op: &crate::op::Op, base: &Path) -> ER
             }
             "pipe-text" => {
                 let s = pipe!();
-                Ok(Value::String(s))
+                Ok(Value::String(s.trim_end().to_string()))
             }
             _ => Ok(Value::String(val)),
         }
